%% estimation of affine transformation
N=10;
P1=randi(100,2,N);  %generate random 2D integer locations
%T=[2 0 0;0 1 0;0 0 1];
T=[1.3 4.8 -1;3.6 5.8 20;0 0 1];
P2=T*[P1;ones(1,size(P1,2))];   %transform to get new locations
estimatedT=recover_affine_transform(P1,P2);
T
estimatedT

% verify correctness of results
% green circles should be around the blue plus signs
P3=estimatedT*[P1;ones(1,size(P1,2))];
scatter(P1(2,:),P1(1,:),150,'r+');
hold on;
scatter(P2(2,:),P2(1,:),150,'b+');
scatter(P3(2,:),P3(1,:),150,'go');
legend('P1','P2','P3','Location','NorthWest');
print(gcf,'affine_transformation','-dpng');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% image warping
I=imread('illusory_square.jpg');
if ndims(I)==3
    I=rgb2gray(I);
end
I(390:400,130:140)=255;
I=double(I);
imwrite(myAffineRescaling(I,1),'illusory_square_with_dot.png');

%setup a 2D affine transformation matrix A
theta=90*pi/180; ct=cos(theta); st=sin(theta); R=[ct -st 0;st ct 0;0 0 1];
Sx=1; Sy=-2; S=[Sx 0 0;0 Sy 0;0 0 1];
Shx=0.5; Shy=0; Sh=[1 Shx 0;Shy 1 0;0 0 1];
Tx=50; Ty=-100; T=[1 0 Tx;0 1 Ty;0 0 1];
A=Sx*Sh*T*R;

%transform image using 2D affine transform A
I1=warp_image(I,A);
clf;
imagesc(I1); axis equal; axis tight; colormap gray
imwrite(myAffineRescaling(I1,1),'illusory_square_affine_warped.png');

%setup a 2D projective transform (homography) H
H=A;
H(3,1)=.001; H(3,2)=.001;

%transform image using 2D projective transform H
I1=warp_image(I,H);
imagesc(I1); axis equal; axis tight; colormap gray
imwrite(myAffineRescaling(I1,1),'illusory_square_projective_warped.png');